<?php
/**
* Output page controller
*
* @copyright 2002-2009 by papaya Software GmbH - All rights reserved.
* @link http://www.papaya-cms.com/
* @license http://www.gnu.org/licenses/old-licenses/gpl-2.0.html GNU General Public License, version 2
*
* You can redistribute and/or modify this script under the terms of the GNU General Public
* License (GPL) version 2, provided that the copyright and license notes, including these
* lines, remain unmodified. papaya is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
* FOR A PARTICULAR PURPOSE.
*
* @package Papaya
* @subpackage Frontend
* @version $Id: papaya_page.php 39558 2014-03-13 13:59:02Z weinert $
*/

/**
* basic class
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_base_object.php');

define('PAPAYA_PAGE_ERROR_UNKNOWN', 0);
define('PAPAYA_PAGE_ERROR_DATABASE', 1);
define('PAPAYA_PAGE_ERROR_ACCESS', 2);
define('PAPAYA_PAGE_ERROR_PATH', 3);

define('PAPAYA_PAGE_ERROR_PAGE', 101);
define('PAPAYA_PAGE_ERROR_PAGE_PUBLIC', 102);
define('PAPAYA_PAGE_ERROR_PAGE_CONTENT', 103);
define('PAPAYA_PAGE_ERROR_PAGE_DOMAIN', 104);
define('PAPAYA_PAGE_ERROR_PAGE_RECURSION', 105);

define('PAPAYA_PAGE_ERROR_BOX', 201);
define('PAPAYA_PAGE_ERROR_BOX_CONTENT', 202);

define('PAPAYA_PAGE_ERROR_VIEW', 301);
define('PAPAYA_PAGE_ERROR_OUTPUT', 302);
define('PAPAYA_PAGE_ERROR_OUTPUT_SETTINGS', 303);

define('PAPAYA_PAGE_ERROR_IMAGE', 401);
define('PAPAYA_PAGE_ERROR_IMAGE_CREATE', 402);
define('PAPAYA_PAGE_ERROR_IMAGE_ACCESS', 403);

define('PAPAYA_PAGE_ERROR_MEDIA', 501);
define('PAPAYA_PAGE_ERROR_MEDIA_ACCESS', 502);
define('PAPAYA_PAGE_ERROR_MEDIA_FILE', 503);
define('PAPAYA_PAGE_ERROR_MEDIA_THUMBNAIL', 504);
define('PAPAYA_PAGE_ERROR_MEDIA_READ', 505);

/**
* Output page
*
* @package Papaya
* @subpackage Frontend
*/
class papaya_page extends base_object {

  private $_isPreview = NULL;

  /**
  * ID of topic
  * @var integer $topicId
  */
  var $topicId = 0;

  /**
  * topic object
  * @var papaya_topic $topic
  */
  var $topic = NULL;

  /**
  * Ouput filter manager, instance of
  * @var papaya_output $output
  */
  var $output = NULL;

  /**
  * Layout
  * @var papaya_xsl $layout
  */
  var $layout = NULL;

  /**
  * Domain handler (vhosts for papayaCMS)
  * @var base_domains $domains
  */
  var $domains = NULL;

  /**
   * The id of the current domain, used from other objects to load domain data later.
   * @var integer
   */
  private $_currentDomainId = 0;

  /**
  * error data
  * array(status => http status, msg => string message, code => error code)
  * @var NULL|array
  */
  var $error = NULL;

  /**
  * redirect data
  * array(status => http status, url => target url)
  * @var NULL|array
  */
  var $redirect = NULL;

  /**
  * Output modus
  * @var string $mode
  */
  var $mode = 'html';

  /**
  * Request data array
  * @var array $requestData
  */
  var $requestData = array();

  /**
  * current visitor language identifier
  * @var string
  */
  var $visitorLanguage = '';

  /**
  * Accept .gz compressed files
  * @var boolean $acceptGzip
  */
  var $acceptGzip = FALSE;

  /**
  * Session Name Suffix
  * @var string
  */
  var $sessionName = '';

  /**
  * Global Session Params
  * @var array
  */
  var $sessionParams = array();

  /**
  * default status for pages is public
  * @var boolean
  */
  var $public = TRUE;

  /**
  * allow session for current output mode
  * @var integer
  */
  var $allowSession = PapayaSession::ACTIVATION_NEVER;
  /**
  * redirect to handle session id in path
  * @var boolean
  */
  var $allowSessionRedirects = FALSE;
  /**
  * session cache mode private or nocache
  * @var string
  */
  var $allowSessionCache = 'private';

  /**
  * current content language data
  * @var string
  */
  var $contentLanguageIdent;

  private $_pageDocument = NULL;
  private $_filterOptions = array();
  private $_boxesList = NULL;

  /**
  * execute page controller
  *
  * @access public
  */
  function execute() {
    PapayaRequestLog::getInstance();
    include_once(PAPAYA_INCLUDE_PATH.'system/Papaya/Autoloader.php');
    spl_autoload_register('PapayaAutoloader::load');
    $application = $this->papaya();
    $application->registerProfiles(
      new PapayaApplicationProfilesCms()
    );
    $application->profiler->start();
    if (!defined('PAPAYA_ADMIN_PAGE')) {
      define('PAPAYA_ADMIN_PAGE', FALSE);
    }
    if (!defined('PAPAYA_WEBSITE_REVISION')) {
      define('PAPAYA_WEBSITE_REVISION', '');
    }
    if (defined('PAPAYA_SESSION_NAME')) {
      $this->sessionName = PAPAYA_SESSION_NAME;
    }
    if (!isset($_SERVER['HTTP_HOST'])) {
      $_SERVER['HTTP_HOST'] = $_SERVER['SERVER_NAME'];
    }
    $options = $application->getObject('Options');
    if (!$options->load()) {
      $controller = new PapayaControllerErrorFile($this);
      $controller->setStatus(503);
      $controller->setError('Service Unavailable', 'DATABASE');
      $controller->setTemplateFile($options->get('PAPAYA_ERRORDOCUMENT_503', ''));
      $controller->execute($this);
      exit;
    }

    include_once(PAPAYA_INCLUDE_PATH.'system/base_domains.php');
    $this->domains = new base_domains();
    $this->domains->handleDomain();
    $this->_currentDomainId = $this->domains->getCurrentId();

    $options->defineDatabaseTables();
    $options->setupPaths();
    $application->messages->setUp($application->options);

    /* handle redirected errors */
    $request = $application->getObject('Request');
    $redirectErrorCode = $request->getParameter(
      'redirect', 0, NULL, PapayaRequest::SOURCE_QUERY
    );
    if (in_array($redirectErrorCode, array(403, 404, 500))) {
      $message = $request->getParameter(
        'msg', '', NULL, PapayaRequest::SOURCE_QUERY
      );
      $code = $request->getParameter(
        'code', 0, NULL, PapayaRequest::SOURCE_QUERY
      );
      $options->defineConstants();
      $this->getError($redirectErrorCode, $message, $code);
      exit;
    }
    $this->requestData = base_object::parseRequestURI();
    if ($this->isPreview()) {
      $this->sessionName .= 'admin';
      define('PAPAYA_ADMIN_SESSION', TRUE);
      if ($options->getOption('PAPAYA_UI_SECURE', FALSE) &&
          $application->request->getUrl()->scheme != 'https') {
        $url = $application->request->getUrl();
        $url->scheme = 'https';
        $this->doRedirect(301, $url->getUrl(), 'Secure administration');
      }
    } else {
      define('PAPAYA_ADMIN_SESSION', FALSE);
    }

    $this->initializeParams();
    if ($options->get('PAPAYA_CONTENT_LANGUAGE_COOKIE', FALSE) &&
        isset($_COOKIE) && is_array($_COOKIE) && isset($_COOKIE['lng']) &&
        preg_match('(^[a-zA-Z]+$)D', $_COOKIE['lng'])) {
      $this->visitorLanguage = $_COOKIE['lng'];
    }
    $this->initPageMode();
    /* exit script handling */
    if (isset($_GET['exit'])) {
      $this->allowSessionRedirects = FALSE;
      $this->startSession();
      $urlPattern = '(^/sid[a-z]+([a-zA-Z\d,-]{20,40})((?:/.*)|$))i';
      if (preg_match($urlPattern, $_SERVER['REQUEST_URI'], $regs)) {
        $this->doRedirectToPath(
          301,
          $regs[2].'?exit='.urlencode($_GET['exit']),
          FALSE,
          'External Link, Remove SID'
        );
      } else {
        $targetUrl = $_GET['exit'];
        $this->logRequestExitPage($targetUrl);
        $this->protectedRedirect(301, $targetUrl);
      }
    } else {
      $this->startSession();
      if ($this->isPreview()) {
        $previewDomain = $application->session->values['PAGE_PREVIEW_DOMAIN'];
        $this->sendHeader('X-Papaya-Preview-Domain: '.$previewDomain);
        if (!empty($previewDomain)) {
          $domainOptions = new PapayaConfigurationStorageDomain($previewDomain);
          $application->options->load($domainOptions);
          if ($domainId = $domainOptions->domain()->id) {
            $this->_currentDomainId = $domainId;
          }
        }
      }
    }
    $options->defineConstants();
    /* redirect script handling */
    if (!empty($_GET['redirect'])) {
      $targetUrl = base_object::getAbsoluteURL(
        $_GET['redirect'],
        empty($_GET['title']) ? '' : $_GET['title']
      );
      $this->protectedRedirect(302, $targetUrl);
      exit;
    } elseif (!empty($_POST['redirect'])) {
      $targetUrl = base_object::getAbsoluteURL(
        $_POST['redirect'],
        empty($_POST['title']) ? '' : $_POST['title']
      );
      $this->protectedRedirect(302, $targetUrl);
      exit;
    }
    if ($options->get('PAPAYA_DEFAULT_HOST', '') != '' &&
        $options->get('PAPAYA_DEFAULT_HOST_ACTION', 0) > 0 &&
        strtolower($_SERVER['HTTP_HOST']) != strtolower($options->get('PAPAYA_DEFAULT_HOST'))) {
      $this->doRedirectToPath(
        301, $_SERVER['REQUEST_URI'], TRUE, 'Redirect To Default Host'
      );
    }
    $this->checkAcceptGzip();
  }

  /**
  * This method performs a protected redirect.
  *
  * 1) it checks whether the target URL contians the system URL
  * 2) it checks whether the referer contains the system URL
  * 3) it checks whether the target URL contains any of the configured domains
  * 4) if any of the above fails, a redirect page will be shown to inform the visitor
  *    about a possibly spoofed redirect
  *
  * @param integer $code the redirection status code to use
  * @param string $targetURL the target url to redirect to
  */
  function protectedRedirect($code, $targetUrl) {
    if ($this->papaya()->options->get('PAPAYA_REDIRECT_PROTECTION', FALSE)) {
      $protocol = PapayaUtilServerProtocol::get();
      $systemUrl = $protocol.'://'.strtolower(
        $this->papaya()->options->get(
          'PAPAYA_DEFAULT_HOST',
          empty($_SERVER['HTTP_HOST']) ? 'localhost' : ''
        )
      );
      if (FALSE !== strpos($targetUrl, "\n") &&
          FALSE !== strpos($targetUrl, "\r")) {
        //don't redirect to prevent header injection
        $this->sendHeader('X-Papaya-Status: redirect link contains newline');
      } elseif (0 === strpos($targetUrl, $systemUrl)) {
        //own hostname - just redirect
        $this->doRedirect($code, $targetUrl, 'Absolute Link (same domain)');
      } elseif (isset($_SERVER['HTTP_REFERER']) &&
                0 === strpos($_SERVER['HTTP_REFERER'], $systemUrl)) {
        //from own hostname - just redirect
        $this->doRedirect($code, $targetUrl, 'External Link (referer checked)');
      } else {
        include_once(PAPAYA_INCLUDE_PATH.'system/base_url_analyze.php');
        $urlData = base_url_analyze::parseURL($targetUrl);
        if ($this->domains->load($urlData['host'], 0)) {
          $this->doRedirect($code, $targetUrl, 'External Link (domain checked)');
        } else {
          $this->getRedirect($code, $targetUrl);
        }
      }
    } else {
      $this->doRedirect($code, $targetUrl, 'Redirect Link');
    }
  }

  /**
  * Redirect to another path of the same installation
  *
  * @param integer $code
  * @param string $path
  * @param boolean $defaultHost
  * @param string $reason
  */
  function doRedirectToPath($code, $path, $defaultHost = FALSE, $reason = NULL) {
    if ($defaultHost &&
        defined('PAPAYA_DEFAULT_HOST') &&
        PAPAYA_DEFAULT_HOST != '') {
      $host = PAPAYA_DEFAULT_HOST;
    } else {
      $host = $_SERVER['HTTP_HOST'];
    }
    $protocol = PapayaUtilServerProtocol::get();
    $targetUrl = $protocol.'://'.strtolower($host).$path;
    $this->doRedirect($code, $targetUrl, $reason);
  }

  /**
  * Send a http redirect
  * @param integer $code
  * @param string $targetUrl
  * @param string $reason
  */
  function doRedirect($code, $targetUrl, $reason = NULL) {
    $lineBreaks = array("\r", "\n");
    $code = ($code == 301) ? $code : 302;
    $this->sendHTTPStatus($code);
    if (!empty($reason)) {
      $this->sendHeader('X-Papaya-Status: '.str_replace($lineBreaks, '', $reason));
    }
    $this->sendHeader('Location: '.str_replace($lineBreaks, '', $targetUrl));
    $this->sendHeader('Expires: '.gmdate('D, d M Y H:i:s', (time() - 31536000)).' GMT');
    $this->sendHeader(
      'Cache-Control: no-store, no-cache, must-revalidate, post-check=0, pre-check=0'
    );
    $this->sendHeader('Pragma: no-cache');
    $this->papaya()->profiler->store();
    exit;
  }

  /**
  * Initialize parameters
  *
  * @access public
  */
  function initializeParams($sessionParamName = NULL) {
    if (isset($_REQUEST['tt']['page_id']) && $_REQUEST['tt']['page_id'] > 0) {
      $this->topicId = (int)$_REQUEST['tt']['page_id'];
    } elseif (isset($_REQUEST['p_id']) && $_REQUEST['p_id'] > 0) {
      $this->topicId = (int)$_REQUEST['p_id'];
    } elseif (isset($this->requestData['page_id']) &&
              $this->requestData['page_id'] > 0) {
      $this->topicId = (int)$this->requestData['page_id'];
      $_REQUEST['p_id'] = (int)$this->requestData['page_id'];
    } else {
      $aliasPage = $this->checkAlias();
      if (TRUE === $aliasPage) {
        $this->topicId = -1;
      } elseif (FALSE === $aliasPage) {
        $this->topicId = $this->papaya()->options->get('PAPAYA_PAGEID_DEFAULT', 0);
      } else {
        $this->topicId = (int)$aliasPage;
      }
      $_REQUEST['p_id'] = $this->topicId;
    }
    if (isset($this->requestData['language'])) {
      $_REQUEST['language'] = $this->requestData['language'];
    }
    if (isset($_GET['__cms_box_id']) && $_GET['__cms_box_id'] > 0) {
      $this->boxId = (int)$_GET['__cms_box_id'];
    } else {
      $this->boxId = -1;
    }
  }

  /**
  * Check alias
  *
  * @return mixed Either FALSE when redirect status is 404, otherwise TRUE.
  * returns int page-ID of alias if the request URI is an alias.
  */
  function checkAlias() {
    if (!empty($_SERVER['REDIRECT_PAPAYA_STATUS'])) {
      $status = (int)$_SERVER['REDIRECT_PAPAYA_STATUS'];
    } else {
      $status = empty($_SERVER['REDIRECT_STATUS']) ? 200 : (int)$_SERVER['REDIRECT_STATUS'];
    }
    if ($status == 404) {
      $options = $this->papaya()->options;
      $options->setupPaths();
      $requestURI = isset($_SERVER['REQUEST_URI']) ? $_SERVER['REQUEST_URI'] : '/';
      $publicFilesPath = $options->get('PAPAYA_PATH_PUBLICFILES', '');
      if (preg_match('(^/?([\\("]|\\\\"))', $requestURI)) {
        /* mac-ie css hack handling - path starts with ( or " */
        $this->sendHTTPStatus(200);
        exit;
      } elseif (preg_match('(^/?blank.gif)', $requestURI)) {
        /* ivw sends a relative location header for blank.gif.
           If a client does not support relative location headers,
           papaya will get the request and send a transparent 1px gif
        */
        $this->sendHTTPStatus(200);
        $this->sendHeader('Content-type: image/gif');
        $this->sendHeader('Expires: '.gmdate('D, d M Y H:i:s', (time() + 2592000)).' GMT');
        // @codingStandardsIgnoreStart
        printf(
          '%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%',
          71, 73, 70, 56, 57, 97, 1, 0, 1, 0, 128, 255, 0, 192, 192, 192, 0, 0, 0, 33,
          249, 4, 1, 0, 0, 0, 0, 44, 0, 0, 0, 0, 1, 0, 1, 0, 0, 2, 2, 68, 1, 0, 59
        );
        // @codingStandardsIgnoreEnd
        exit;
      } elseif (preg_match('(favicon\.ico)i', $requestURI)) {
        /* if we get an error for a favicon.ico - let's look in the theme */
        $webFile = 'papaya-themes/'.$options->get('PAPAYA_LAYOUT_THEME', 'default').'/favicon.ico';
        $localFile = $this->getBasePath(TRUE).$webFile;
        if (file_exists($localFile) && is_readable($localFile)) {
          $fileData = array(
            'file_name' => 'favicon.ico',
            'file_size' => filesize($localFile),
            'mimetype' => 'image/x-icon'
          );
          $this->sendHTTPStatus(200);
          include_once(PAPAYA_INCLUDE_PATH.'system/papaya_file_delivery.php');
          papaya_file_delivery::outputFile($localFile, $fileData);
          exit;
        } else {
          $this->getError(
            404,
            'File/path '.$_SERVER['REQUEST_URI'].' not found',
            $options->get('PAPAYA_PAGE_ERROR_PATH', 0)
          );
        }
        exit;
      } elseif (!empty($publicFilesPath) && 0 === strpos($requestURI, $publicFilesPath)) {
        $options->defineConstants();
        $this->getMediaThumbFile(substr($requestURI, strrpos($requestURI, '/') + 1));
        exit;
      } else {
        $themeWrapperUrl = new PapayaThemeWrapperUrl();
        switch ($themeWrapperUrl->getMimetype()) {
        case 'text/javascript' :
        case 'text/css' :
          $themeWrapper = new PapayaThemeWrapper($themeWrapperUrl);
          $themeWrapper->getResponse()->send();
          exit;
        }

        include_once(PAPAYA_INCLUDE_PATH.'system/base_urlmounter.php');
        $urlMounter = new base_urlmounter;
        $this->allowSessionRedirects = FALSE;
        if ($alias = $urlMounter->locate()) {
          switch ($alias[1]) {
          case 3 :
            $this->doRedirect(302, $alias[0], 'Alias Redirect');
            return TRUE;
          case 2 :
            if ($url = $urlMounter->executeAliasPlugin($alias[2])) {
              if (is_string($url)) {
                $url = $this->getAbsoluteUrl($url);
                if ($aliasUrl = $urlMounter->getAliasURL($url)) {
                  $this->doRedirect(302, $aliasUrl, 'Alias Plugin Redirect');
                } else {
                  $requestUrl = new PapayaUrl($url);
                  $request = new PapayaRequest($this->papaya()->options);
                  $request->load($requestUrl);
                  $urlData = array(
                    'topic_id' => $request->getParameter(
                      'page_id', NULL, NULL, PapayaRequest::SOURCE_PATH
                    ),
                    'lng_ident' => $request->getParameter(
                      'language', NULL, NULL, PapayaRequest::SOURCE_PATH
                    ),
                    'viewmode_ext' => $request->getParameter(
                      'output_mode', NULL, NULL, PapayaRequest::SOURCE_PATH
                    ),
                    'url_params' => $requestUrl->getQuery(),
                  );
                  return $this->setRequestFromAlias($urlData);
                }
              } elseif (is_array($url)) {
                return $this->setRequestFromAlias($url);
              } elseif ($url === TRUE) {
                //url was handled in plugin
                exit;
              }
            }
            break;
          case 1 :
            $this->sendHTTPStatus();
            $this->sendHeader('Content-type: text/html; charset=utf-8');
            echo $urlMounter->getOutput($alias[0], $alias[2]);
            exit;
          default :
            if ($aliasUrl = $urlMounter->getAliasURL($alias[0])) {
              $this->doRedirect(302, $aliasUrl, 'Alias Redirect');
            } else {
              //no redirect needed - put data into variables
              return $this->setRequestFromAlias($alias[2]);
            }
          }
        }
      }
      return TRUE;
    }
    return FALSE;
  }

  /**
  * Initialize request data from alias configuration
  * @param array $alias
  * @return integer page id
  */
  function setRequestFromAlias($alias) {
    define('PAPAYA_ALIAS_PAGE', TRUE);
    $this->sendHTTPStatus();
    $application = $this->papaya();
    $parameterGroupSeparator = $application
      ->options
      ->getOption('PAPAYA_URL_LEVEL_SEPARATOR');
    $request = $application->request;
    $parameters = $request->getParameters(PapayaRequest::SOURCE_QUERY);
    $query = new PapayaRequestParametersQuery($parameterGroupSeparator);
    //get addtional parameters and merge them
    if (!empty($_SERVER['REDIRECT_QUERY_STRING'])) {
      $parameters->merge(
        $query->setString($_SERVER['REDIRECT_QUERY_STRING'])->values()
      );
    }
    if (!empty($alias['url_params'])) {
      $parameters->merge(
        $query->setString($alias['url_params'])->values()
      );
    }
    $reference = new PapayaUiReferencePage();
    if (isset($alias['topic_id'])) {
      $reference->setPageId($alias['topic_id']);
    }
    if (isset($alias['lng_ident'])) {
      $reference->setPageLanguage($alias['lng_ident']);
    }
    if (isset($alias['viewmode_ext'])) {
      $reference->setOutputMode($alias['viewmode_ext']);
      $this->mode = $alias['viewmode_ext'];
    }
    $reference->setParameters($parameters);
    //exchange request object
    $request = new PapayaRequest($application->options);
    $request->load(new PapayaUrl($reference->get()));
    $request->setParameters(PapayaRequest::SOURCE_QUERY, $parameters);
    $application->setObject(
      'Request', $request, PapayaApplication::DUPLICATE_OVERWRITE
    );
    // bc stuff
    $_GET = $request->getParameters(PapayaRequest::SOURCE_QUERY)->toArray();
    $requestData['page_id'] = $alias['topic_id'];
    $requestData['language'] = $alias['lng_ident'];
    $requestData['ext'] = $alias['viewmode_ext'];
    $this->requestData = PapayaUtilArray::merge(
      $this->requestData,
      $requestData
    );
    return $alias['topic_id'];
  }

  /**
  * Convert a query string into an array
  * @param string $queryString
  * @return array
  */
  function queryStringToArray($queryString) {
    $query = new PapayaRequestParametersQuery(
      $this->papaya()->options->getOption('PAPAYA_URL_LEVEL_SEPARATOR', '')
    );
    return $query->setString($queryString)->values()->toArray();
  }

  /**
  * Start new session
  *
  * @access public
  */
  function startSession() {
    $session = $this->papaya()->session;
    if ($this->isPreview()) {
      $startSession = PapayaSession::ACTIVATION_ALWAYS;
    } elseif ($this->allowSession &&
              $this->papaya()->options->get('PAPAYA_SESSION_START', FALSE)) {
      $startSession = $this->allowSession;
    } else {
      $startSession = PapayaSession::ACTIVATION_NEVER;
    }
    $session->setName('sid'.$this->sessionName);
    $this->sendHeader('P3P: CP="NOI NID ADMa OUR IND UNI COM NAV"');
    if (PapayaUtilServerAgent::isRobot()) {
      if ($redirect = $session->redirectIfNeeded()) {
        $redirect->send();
        exit;
      }
    } elseif (($startSession == PapayaSession::ACTIVATION_ALWAYS) ||
              ($startSession == PapayaSession::ACTIVATION_DYNAMIC && $session->id()->existsIn())) {
      if ($this->papaya()->options->get('PAPAYA_SESSION_START', FALSE) &&
          !$this->papaya()->options->get('PAPAYA_DB_CONNECT_PERSISTENT', FALSE)) {
        $this->output->databaseClose();
      }
      $fallback = $this->papaya()->options->get('PAPAYA_SESSION_ID_FALLBACK', 'rewrite');
      switch ($fallback) {
      case 'get' :
        $session->options()->fallback = PapayaSessionOptions::FALLBACK_PARAMETER;
        break;
      case 'rewrite' :
        $session->options()->fallback = PapayaSessionOptions::FALLBACK_REWRITE;
        break;
      default :
        if ($this->isPreview()) {
          $session->options()->fallback = PapayaSessionOptions::FALLBACK_REWRITE;
        } else {
          $session->options()->fallback = PapayaSessionOptions::FALLBACK_NONE;
        }
      }
      $session->options()->cache = $this->papaya()->options->get(
        'PAPAYA_SESSION_CACHE', PapayaSessionOptions::CACHE_NONE
      );
      if ($redirect = $session->activate($this->allowSessionRedirects)) {
        $redirect->send();
        exit;
      }
      if ($this->readOnlySession) {
        $session->close();
      }
      $this->sessionParams = $session->values['PAPAYA_SESSION_PAGE_PARAMS'];
    }
  }

  /**
  * Set page option
  *
  * @param string $name
  * @param mixed $value
  * @access public
  */
  function setPageOption($name, $value) {
    if (isset($value)) {
      $this->sessionParams[$name] = $value;
    } else {
      unset($this->sessionParams[$name]);
    }
    $this->papaya()->session->values['PAPAYA_SESSION_PAGE_PARAMS'] = $this->sessionParams;
  }

  /**
  * Get page option
  *
  * @param string $name
  * @access public
  * @return mixed parameter value or NULL
  */
  function getPageOption($name) {
    if (is_array($this->sessionParams) &&
        isset($this->sessionParams[$name])) {
      return $this->sessionParams[$name];
    } else {
      return NULL;
    }
  }

  /**
  * Is access valid
  *
  * @param integer $topicId
  * @access public
  * @return boolean
  */
  function validateAccess($topicId) {
    if ($this->isPreview()) {
      return $this->validateEditorAccess();
    } else {
      return ($this->papaya()->surfer->canView($topicId));
    }
  }

  /**
  * preview or part of current domain
  *
  * @access public
  * @return boolean
  */
  function validateDomain() {
    if ($this->isPreview()) {
      return TRUE;
    } elseif (defined('PAPAYA_PAGEID_DOMAIN_ROOT') && PAPAYA_PAGEID_DOMAIN_ROOT > 0) {
      $result = $this->topic->hasParent(PAPAYA_PAGEID_DOMAIN_ROOT);
    } else {
      $result = TRUE;
    }
    if ($result && $this->topic->topic['topic_protocol'] > 0) {
      $protocol = PapayaUtilServerProtocol::isSecure() ? 2 : 1;
      if ($protocol != $this->topic->topic['topic_protocol']) {
        $targetUrl = $this->topic->topic['topic_protocol'] == 2 ? 'https://' : 'http://';
        $targetUrl .= empty($_SERVER['HTTP_HOST']) ? 'localhost' : $_SERVER['HTTP_HOST'];
        $targetUrl .= empty($_SERVER['REQUEST_URI']) ? '/' : $_SERVER['REQUEST_URI'];
        $this->protectedRedirect('302', $targetUrl);
      }
    } elseif ($result && defined('PAPAYA_DEFAULT_PROTOCOL') && PAPAYA_DEFAULT_PROTOCOL > 0) {
      $protocol = PapayaUtilServerProtocol::isSecure() ? 2 : 1;
      if ($protocol != PAPAYA_DEFAULT_PROTOCOL) {
        $targetUrl = PAPAYA_DEFAULT_PROTOCOL == 2 ? 'https://' : 'http://';
        $targetUrl .= empty($_SERVER['HTTP_HOST']) ? 'localhost' : $_SERVER['HTTP_HOST'];
        $targetUrl .= empty($_SERVER['REQUEST_URI']) ? '/' : $_SERVER['REQUEST_URI'];
        $this->protectedRedirect('302', $targetUrl);
      }
    }
    return $result;
  }

  /**
  * validate editor access (preview, debug outputs)
  *
  * @access public
  * @return boolean
  */
  function validateEditorAccess() {
    $application = &$this->papaya();
    $GLOBALS['PAPAYA_USER'] = &$application->getObject('AdministrationUser');
    if (is_null($GLOBALS['PAPAYA_USER']->isValid)) {
      $GLOBALS['PAPAYA_USER']->initialize();
      $GLOBALS['PAPAYA_USER']->execLogin();
    }
    return ($GLOBALS['PAPAYA_USER']->isValid);
  }


  /**
  * set visitor language parameter cookie
  * @param string $lngString
  * @return void
  */
  function setVisitorLanguage($lngString) {
    if (defined('PAPAYA_CONTENT_LANGUAGE_COOKIE') && PAPAYA_CONTENT_LANGUAGE_COOKIE) {
      if (defined('PAPAYA_SESSION_DOMAIN') && PAPAYA_SESSION_DOMAIN != '') {
        $domain = PAPAYA_SESSION_DOMAIN;
      } else {
        $domain = NULL;
      }
      if (defined('PAPAYA_SESSION_PATH') && PAPAYA_SESSION_PATH != '') {
        $path = PAPAYA_SESSION_PATH;
      } else {
        $path = '/';
      }
      setcookie('lng', urlencode($lngString), NULL, $path, $domain);
    }
  }

  public function createPage() {
    if ($this->isPreview()) {
      include_once(PAPAYA_INCLUDE_PATH.'system/papaya_topic.php');
      return new papaya_topic();
    } else {
      include_once(PAPAYA_INCLUDE_PATH.'system/papaya_publictopic.php');
      return new papaya_publictopic();
    }
  }

  /**
  * Get cache identifier
  *
  * @access public
  * @return FALSE|string $cacheId
  */
  function getCacheId() {
    $definition = NULL;
    $pagePlugin = $this->papaya()->plugins->get(
      $this->topic->topic['TRANSLATION']['module_guid'],
      NULL,
      $this->topic->topic['TRANSLATION']['topic_content']
    );
    if ($pagePlugin && ($boxesList = $this->getBoxes())) {
      if ($pagePlugin instanceOf PapayaPluginCacheable) {
        $definition = $pagePlugin->cacheable();
      } elseif (empty($pagePlugin->cacheable) || $pagePlugin->cacheable == FALSE) {
        return new PapayaCacheIdentifierDefinitionBoolean(FALSE);
      } elseif (method_exists($pagePlugin, 'getCacheId')) {
        $definition = new PapayaCacheIdentifierDefinitionCallback(array($pagePlugin, 'getCacheId'));
      } else {
        $definition = new PapayaCacheIdentifierDefinitionBoolean(TRUE);
      }
      $definition = new PapayaCacheIdentifierDefinitionGroup(
        new PapayaCacheIdentifierDefinitionBoolean(PapayaUtilRequestMethod::isGet()),
        new PapayaCacheIdentifierDefinitionUrl(),
        new PapayaCacheIdentifierDefinitionSurfer(),
        new PapayaCacheIdentifierDefinitionSessionParameters('PAPAYA_SESSION_PAGE_PARAMS'),
        $definition,
        $boxesList->cacheable()
      );
      if ($status = $definition->getStatus()) {
        return 'page_'.$this->topicId.'_'.md5(serialize($status));
      }
    }
    return FALSE;
  }

  /**
  * Use generic output cache
  *
  * @access public
  * @return boolean
  */
  function useCache() {
    $method = empty($_SERVER['REQUEST_METHOD']) ? '' : strtoupper($_SERVER['REQUEST_METHOD']);
    $agent = empty($_SERVER['HTTP_USER_AGENT']) ? '' : strtolower($_SERVER['HTTP_USER_AGENT']);
    return (
      !$this->isPreview() &&
      $method == 'GET' &&
      PAPAYA_CACHE_OUTPUT &&
      PAPAYA_CACHE_TIME_OUTPUT > 0 &&
      $agent != 'mnogosearch-dimensional'
    );
  }

  /**
  * Get cache
  *
  * @param integer $cacheId
  * @access public
  * @return boolean
  */
  function getCache($cacheId) {
    if (defined('PAPAYA_CACHE_OUTPUT') && PAPAYA_CACHE_OUTPUT &&
        defined('PAPAYA_CACHE_TIME_OUTPUT') && PAPAYA_CACHE_TIME_OUTPUT > 0) {
      if (isset($_SERVER['HTTP_IF_NONE_MATCH'])) {
        if ('"'.$cacheId.'"' == $_SERVER['HTTP_IF_NONE_MATCH'] ||
            $cacheId == $_SERVER['HTTP_IF_NONE_MATCH']) {
          if (isset($_SERVER['HTTP_IF_MODIFIED_SINCE']) &&
              time() < (
                strtotime($_SERVER['HTTP_IF_MODIFIED_SINCE']) + (int)PAPAYA_CACHE_TIME_OUTPUT
              )) {
            $this->sendHTTPStatus(304);
            /* We set the session cookie earlier, thus set the lng cookie.
               This is just to be safe, so the client (e.g. a proxy) doesn't
               forget it from his cached version.
             */
            $this->setVisitorLanguage($this->topic->currentLanguage['lng_ident']);
            /* Etag, Expires and Cache-Control were set before and the client
               already has all the other headers from his cached version
             */
            exit;
          }
        }
      }
      $cache = PapayaCache::getService($this->papaya()->options);
      $cacheIdGzip = $cacheId.'.gz';
      $contentGzip = FALSE;
      if ($this->acceptGzip &&
          defined('PAPAYA_COMPRESS_CACHE_OUTPUT') &&
          PAPAYA_COMPRESS_CACHE_OUTPUT) {
        //compressed output
        $contentGzip = $cache->read(
          'output',
          $this->topicId,
          '.'.$cacheIdGzip,
          PAPAYA_CACHE_TIME_OUTPUT
        );
        $mtime = $cache->created(
          'output',
          $this->topicId,
          '.'.$cacheIdGzip,
          PAPAYA_CACHE_TIME_OUTPUT
        );
        if ($contentGzip && FALSE !== $mtime && !headers_sent()) {
          $this->sendHTTPStatus(200);
          $this->setVisitorLanguage($this->topic->currentLanguage['lng_ident']);
          $this->output->sendHeader();
          $this->sendHeader('Last-Modified: '.gmdate('D, d M Y H:i:s', $mtime).' GMT');
          $this->sendHeader('Content-Encoding: gzip');
          $this->sendHeader('X-Papaya-Gzip: yes');
          $this->sendHeader('X-Papaya-Cache: yes');
          $this->sendHeader('Content-Length: '.strlen($contentGzip));
          echo $contentGzip;
          flush();
          return TRUE;
        }
      } else {
        //uncompressed output
        $content = $cache->read(
          'output',
          $this->topicId,
          '.'.$cacheId,
          PAPAYA_CACHE_TIME_OUTPUT
        );
        $mtime = $cache->created(
          'output',
          $this->topicId,
          '.'.$cacheId,
          PAPAYA_CACHE_TIME_OUTPUT
        );
        if ($content && FALSE !== $mtime) {
          $this->sendHTTPStatus(200);
          $this->setVisitorLanguage($this->topic->currentLanguage['lng_ident']);
          $this->output->sendHeader();
          $this->sendHeader('Last-Modified: '.gmdate('D, d M Y H:i:s', $mtime).' GMT');
          $this->sendHeader('X-Papaya-Cache: yes');
          $this->sendHeader('Content-Length: '.strlen($content));
          echo $content;
          flush();
          return TRUE;
        }
      }
    }
    return FALSE;
  }

  /**
  * Set cache
  *
  * @param integer $cacheId
  * @param string $page
  * @access public
  * @return boolean
  */
  function setCache($cacheId, $topicId, $page) {
    if (defined('PAPAYA_CACHE_OUTPUT') && PAPAYA_CACHE_OUTPUT &&
        defined('PAPAYA_CACHE_TIME_OUTPUT') && PAPAYA_CACHE_TIME_OUTPUT > 0) {
      $cache = PapayaCache::getService($this->papaya()->options);
      if (defined('PAPAYA_COMPRESS_CACHE_OUTPUT') &&
          PAPAYA_COMPRESS_CACHE_OUTPUT) {
        $cache->write(
          'output',
          $topicId,
          '.'.$cacheId.'.gz',
          gzencode($page),
          PAPAYA_CACHE_TIME_OUTPUT
        );
      }
      $cache->write('output', $topicId, '.'.$cacheId, $page, PAPAYA_CACHE_TIME_OUTPUT);
      return TRUE;
    }
    return FALSE;
  }

  /**
  * Initialize page mode
  *
  * @access public
  */
  function initPageMode() {
    $this->versionDateTime = 0;
    if (isset($_GET['p_date'])) {
      $this->versionDateTime = (int)$_GET['p_date'];
    } elseif (isset($this->requestData['datetime']) &&
              $this->requestData['datetime'] > 0) {
      $this->versionDateTime = (int)$this->requestData['datetime'];
    }
    if (!$this->isPreview()) {
      $this->versionDateTime = 0;
    }
    if (!isset($this->requestData['ext'])) {
      $this->requestData['ext'] = 'php';
    }
    switch ($this->requestData['ext']) {
    case 'xml':
      $this->mode = 'xml';
      break;
    case 'php':
      $this->mode = $this->papaya()->options->get('PAPAYA_URL_EXTENSION', 'html');
      break;
    default:
      $this->mode = $this->requestData['ext'];
    }

    include_once(PAPAYA_INCLUDE_PATH.'system/papaya_output.php');
    unset($this->output);
    $this->output = new papaya_output;

    $pageModes = array(
      'urls', 'status', 'thumb', 'thumbnail', 'media', 'popup', 'download', 'image'
    );
    if (isset($this->mode) && in_array($this->mode, $pageModes)) {
      $this->readOnlySession = ($this->mode != 'image');
      $this->allowSession = PapayaSession::ACTIVATION_DYNAMIC;
      $this->allowSessionRedirects = FALSE;
      $this->allowSessionCache = 'private';
    } elseif ($this->output->loadViewModeData($this->mode)) {
      $this->readOnlySession = FALSE;
      if ($this->isPreview()) {
        $pageStatus = new PapayaContentPageStatus();
      } else {
        $pageStatus = new PapayaContentPagePublicationStatus();
      }
      $pageStatus->load($this->topicId);
      if ($pageStatus->sessionMode == 0) {
        $this->allowSession = $this->papaya()->options->getOption(
          'PAPAYA_SESSION_ACTIVATION', PapayaSession::ACTIVATION_ALWAYS
        );
      } else {
        $this->allowSession = $pageStatus->sessionMode;
      }
      if ($this->allowSession != PapayaSession::ACTIVATION_NEVER) {
        if ($this->papaya()->options->get('PAPAYA_SESSION_CACHE') == 'nocache') {
          $this->allowSessionCache = 'nocache';
        } else {
          $this->allowSessionCache = 'private';
        }
        $this->allowSessionRedirects =
          (bool)$this->output->viewMode['viewmode_sessionredirect'];
        switch ($this->output->viewMode['viewmode_sessionmode']) {
        case 1 :
          //read only session
          $this->allowSession = PapayaSession::ACTIVATION_DYNAMIC;
          $this->readOnlySession = TRUE;
          break;
        case 2 :
          //no session
          $this->allowSession = PapayaSession::ACTIVATION_NEVER;
          $this->readOnlySession = TRUE;
          break;
        }
      }
    } else {
      $this->allowSession = PapayaSession::ACTIVATION_DYNAMIC;
      $this->readOnlySession = FALSE;
      $this->allowSessionRedirects = TRUE;
    }
  }

  /**
  * Get request result
  *
  * @access public
  * @return string '' default
  */
  function get() {
    $this->sendHeader('X-Generator: papaya CMS');
    switch ($this->mode) {
    case 'urls':
      $this->getUrls();
      break;
    case 'status':
      $this->getStatus();
      break;
    case 'image':
      $controller = new PapayaControllerImage();
      $this->executeController($controller);
      break;
    case 'thumb':
    case 'thumbnail':
      $this->getMediaThumbFile();
      break;
    case 'media':
      $this->getMediaFile();
      break;
    case 'popup':
      $this->getMediaPopup();
      break;
    case 'download':
      $this->outputDownload();
      break;
    case 'outputs' :
      $this->getOutputs();
      break;
    case 'xml':
      $this->getXMLOutput();
      break;
    default:
      $this->getPageOutput();
      break;
    }
    $this->papaya()->profiler->store();
  }

  /**
  * Get URLs
  *
  * @access public
  */
  function getUrls() {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_sitemap.php');
    $this->topic = $this->createPage();
    $this->map = new base_sitemap(
      $this->topic,
      array('root' => 0, 'format' => 'static', 'forstart' => 0, 'forend' => 999)
    );
    $this->sendHeader('Content-type: text/html');
    echo $this->map->getUrls();
  }

  /**
  * get the current status for the web page
  *
  * currently checks the database connection only
  *
  * @access public
  */
  function getStatus() {
    $database = $this->papaya()->getObject('Database')->getConnector();
    $allStatus = TRUE;
    if (@$database->connect($this, TRUE)) {
      $status['DATABASE'] = TRUE;
    } else {
      $status['DATABASE'] = FALSE;
      $allStatus = FALSE;
    }
    if (@$database->connect($this, FALSE)) {
      $status['DATABASE_WRITE'] = TRUE;
    } else {
      $status['DATABASE_WRITE'] = FALSE;
      $allStatus = FALSE;
    }
    $paths = array(
      'PATH_CACHE' => 'PAPAYA_PATH_CACHE',
      'PATH_FILES' => 'PAPAYA_PATH_MEDIAFILES',
      'PATH_THUMBNAILS' => 'PAPAYA_PATH_THUMBFILES'
    );
    foreach ($paths as $name => $path) {
      $realPath = $this->papaya()->options->get($path, '');
      if (empty($realPath)) {
        $status[$name] = FALSE;
        $allStatus = FALSE;
        continue;
      }
      $directory = new PapayaFileSystemDirectory($realPath);
      if (!($status[$name] = $directory->isWriteable())) {
        $allStatus = FALSE;
      }
    }
    $this->sendHeader('Content-type: text/xml');
    $result = new PapayaXmlDocument();
    $cms = $result->appendElement('cms', array('status' => $allStatus ? 'OK' : 'ERROR'));
    foreach ($status as $option => $value) {
      $cms->appendElement(
        'option', array('name' => $option, 'status' => ($value ? 'OK' : 'ERROR'))
      );
    }
    echo $result->saveXML();
  }


  /**
  * Get page xml
  *
  * @see papaya_xsl::xml
  * @access public
  * @return string xml or ''
  */
  function getPageXML() {
    $loaded = $this->topic->loadOutput(
      $this->topicId, $this->requestData['language'], $this->versionDateTime
    );
    if ($loaded) {
      if ($this->validateDomain()) {
        if ($this->topic->checkPublishPeriod($this->topicId)) {
          if ($this->validateAccess($this->topicId)) {
            $sandbox = $this->papaya()->messages->encapsulate(array($this, 'generatePage'));
            if (call_user_func($sandbox)) {
              $this->sendHTTPStatus();
              $this->sendHeader('Content-type: text/xml; charset=utf-8');
              return $this->layout->xml();
            }
            $this->getError(500, 'Service Unavailable', PAPAYA_PAGE_ERROR_PAGE);
          } else {
            $this->getError(403, 'Access forbidden', PAPAYA_PAGE_ERROR_ACCESS);
          }
        } else {
          $this->getError(
            404,
            sprintf('Page #%d currently not published', $this->topicId),
            PAPAYA_PAGE_ERROR_PAGE_PUBLIC
          );
        }
      } else {
        $this->getError(
          404,
          'Page not in domain root',
          PAPAYA_PAGE_ERROR_PAGE_DOMAIN
        );
      }
    } else {
      $this->getError(404, 'Page not found', PAPAYA_PAGE_ERROR_PAGE);
    }
    return '';
  }

  /**
  * get page meta data only
  *
  * @access public
  * @return string xml or ''
  */
  function getPageMetaXML() {
    $loaded = $this->topic->loadOutput(
      $this->topicId, $this->requestData['language'], $this->versionDateTime
    );
    if ($loaded) {
      if ($this->validateDomain()) {
        if ($this->topic->checkPublishPeriod($this->topicId)) {
          if ($this->validateAccess($this->topicId)) {
            $sandbox = $this->papaya()->messages->encapsulate(array($this, 'generatePage'));
            if (call_user_func($sandbox)) {
              $this->sendHTTPStatus();
              $this->sendHeader('Content-type: text/xml; charset=utf-8');
              return $this->layout->xml();
            }
            $this->getError(500, 'Service Unavailable', PAPAYA_PAGE_ERROR_PAGE);
          } else {
            $this->getError(403, 'Access forbidden', PAPAYA_PAGE_ERROR_ACCESS);
          }
        } else {
          $this->getError(
            404,
            sprintf('Page #%d currently not published', $this->topicId),
            PAPAYA_PAGE_ERROR_PAGE_PUBLIC
          );
        }
      } else {
        $this->getError(
          404,
          'Page not in domain root',
          PAPAYA_PAGE_ERROR_PAGE_DOMAIN
        );
      }
    } else {
      $this->getError(404, 'Page not found', PAPAYA_PAGE_ERROR_PAGE);
    }
    return '';
  }

  /**
  * Get page
  *
  * @access public
  * @return string $str or ''
  */
  function getPage() {
    $this->contentLanguage = $this->topic->currentLanguage;
    if (!empty($this->requestData['language']) &&
      $this->contentLanguage['lng_ident'] != $this->requestData['language']) {
      $this->doRedirect(
        301,
        $this->getAbsoluteURL(
          $this->getWebLink(
            NULL,
            $this->contentLanguage['lng_ident'],
            NULL,
            $this->papaya()->getObject('Request')->getParameters(
              PapayaRequest::SOURCE_QUERY
            ),
            NULL,
            $this->topic->topic['TRANSLATION']['topic_title']
          )
        ),
        'Invalid language'
      );
    }
    if ($this->validateDomain()) {
      if ($this->topic->checkPublishPeriod($this->topicId)) {
        if ($this->validateAccess($this->topicId)) {
          $viewId = $this->topic->getViewId();
          if ($viewId > 0) {
            if ($this->filter = $this->output->getFilter($viewId)) {
              $sandbox = $this->papaya()->messages->encapsulate(array($this, 'generatePage'));
              if (!call_user_func($sandbox, $this->filter->data)) {
                $this->getError(500, 'Service Unavailable', PAPAYA_PAGE_ERROR_PAGE);
                return '';
              }
              if ($this->filter->checkConfiguration()) {
                $str = $this->filter->parsePage($this->topic, $this->layout);
                $this->sendHTTPStatus();
                $this->sendHeader('Last-modified: '.gmdate('D, d M Y H:i:s').' GMT');
                $this->output->sendHeader();
                $application = $this->papaya();
                $application->session->close();
                if ($application->options->getOption('PAPAYA_LOG_RUNTIME_REQUEST', FALSE)) {
                  PapayaRequestLog::getInstance()->logTime('Page generated');
                  PapayaRequestLog::getInstance()->omit(FALSE);
                }
                $this->sendHeader('X-Papaya-Cache: no');
                if ($this->acceptGzip  &&
                    $this->papaya()->options->get('PAPAYA_COMPRESS_OUTPUT', FALSE)) {
                  if ($this->sendHeader('Content-Encoding: gzip')) {
                    $this->sendHeader('X-Papaya-Gzip: yes');
                    $compressed = gzencode($str);
                    $this->sendHeader('Content-Length: '.strlen($compressed));
                    echo $compressed;
                  } else {
                    $this->sendHeader('X-Papaya-Gzip: no');
                    $this->sendHeader('Content-Length: '.strlen($str));
                    echo $str;
                  }
                } else {
                  $this->sendHeader('X-Papaya-Gzip: disabled');
                  $this->sendHeader('Content-Length: '.strlen($str));
                  echo $str;
                }
                if ($application->options->getOption('PAPAYA_LOG_RUNTIME_REQUEST', FALSE)) {
                  PapayaRequestLog::getInstance()->logTime('Page delivered');
                }
                flush();
                $this->logRequest($this->topic->getContentLanguageId());
                return $str;
              } else {
                $this->getError(
                  $this->filter->errorStatus,
                  'Output mode "'.
                    papaya_strings::escapeHTMLChars(basename($this->mode)).
                    '"for page #'.$this->topicId.
                    ' not available: '.$this->filter->errorMessage,
                  PAPAYA_PAGE_ERROR_OUTPUT_SETTINGS
                );
              }
            } else {
              $this->getError(
                500,
                'Output mode "'.
                  papaya_strings::escapeHTMLChars(basename($this->mode)).'" for page #'.
                  $this->topicId.' not found',
                PAPAYA_PAGE_ERROR_OUTPUT
              );
            }
          } else {
            $this->getError(
              500,
                'View "'.
                papaya_strings::escapeHTMLChars(basename($this->mode)).'" for page #'.
                $this->topicId.' not found',
              PAPAYA_PAGE_ERROR_VIEW
            );
          }
        } else {
          $this->getError(403, 'Access forbidden', PAPAYA_PAGE_ERROR_ACCESS);
        }
      } else {
        $this->getError(
          404,
          sprintf('Page #%d currently not published', (int)$this->topicId),
          PAPAYA_PAGE_ERROR_PAGE_PUBLIC
        );
      }
    } else {
      $this->getError(
        404,
        'Page not in domain root',
        PAPAYA_PAGE_ERROR_PAGE_DOMAIN
      );
    }
    return '';
  }

  /**
  * Send out the Expires and Cache-Control header.
  *
  * @param integer $expires time until the request should expire in the browser
  */
  private function sendCacheHeaders($expires) {
    if ($expires > 0) {
      $this->sendHeader(
        sprintf(
          'Expires: %s GMT',
          gmdate('D, d M Y H:i:s', time() + (int)$expires)
        )
      );
      $this->sendHeader(
        sprintf(
          'Cache-Control: %s, max-age=%d, pre-check=%d, no-transform',
          $this->papaya()->session->isActive() ? 'private' : 'public',
          $expires,
          $expires
        )
      );
      $this->sendHeader('Pragma: ');
    } else {
      $this->sendHeader(
        sprintf(
          'Expires: %s GMT',
          gmdate('D, d M Y H:i:s', time() - 5184000)
        )
      );
      $this->sendHeader(
        'Cache-Control: no-store, no-cache, must-revalidate, no-transform'
      );
    }
  }

  /**
  * check url path level difference between cms root and request url
  *
  * @return string|FALSE return root url if it is different
  */
  function checkURLPathLevel() {
    $pageUrl = $this->getWebLink();
    if (FALSE !== strpos($pageUrl, '/')) {
      return $this->getAbsoluteUrl($pageUrl);
    }
    return FALSE;
  }

  /**
  * Generate page
  *
  * @access public
  * @return string
  */
  function generatePage($filterParams = NULL, $outputContent = TRUE, $allowRedirect = TRUE) {
    include_once(PAPAYA_INCLUDE_PATH.'system/papaya_xsl.php');
    $this->layout = new papaya_xsl();

    if (isset($this->output->viewMode) && !empty($this->output->viewMode['viewmode_ext'])) {
      $currentViewMode = $this->output->viewMode['viewmode_ext'];
      if ($this->papaya()->options->get('PAPAYA_URL_EXTENSION') != '') {
        $defaultViewMode = PAPAYA_URL_EXTENSION;
      } else {
        $defaultViewMode = 'php';
      }
    } elseif ($this->papaya()->options->get('PAPAYA_URL_EXTENSION') != '') {
      $currentViewMode = $this->papaya()->options->get('PAPAYA_URL_EXTENSION');
      $defaultViewMode = $this->papaya()->options->get('PAPAYA_URL_EXTENSION');
    } else {
      $currentViewMode = 'php';
      $defaultViewMode = 'php';
    }
    $filterParams['viewmode'] = $currentViewMode;
    $this->_filterOptions = $filterParams;
    if ($outputContent) {
      $document = $this->getPageDocument();
      if ($document->documentElement) {
        $this->layout->add($this->getPageDocument(), 'content');
      }
      if (!(defined('PAPAYA_ALIAS_PAGE') && PAPAYA_ALIAS_PAGE) &&
          $allowRedirect &&
          (
           $url = $this->topic->checkURLFileName(
             $this->papaya()->request->getParameter(
               'page_title', '', NULL, PapayaRequest::SOURCE_PATH
             ),
             $filterParams['viewmode']
           )
         )) {
        $this->doRedirect(301, $url, 'URL Title Fixation');
      } elseif ($allowRedirect && $url = $this->checkURLPathLevel()) {
        $this->doRedirect(301, $url, 'URL Path Fixation');
      }
      $this->layout->add($this->topic->getMetaInfos(), 'meta');
    }
    $this->layout->add(
      $this->output->getViewsList(
        $this->topic->getViewId(),
        $this->topic->topicId,
        $this->topic->topic['TRANSLATION']['topic_title']
      ),
      'views'
    );
    $this->layout->add(
      $this->topic->getTranslationsData($this->topic->topic['TRANSLATION']['lng_id']),
      'translations'
    );
    $this->setVisitorLanguage($this->topic->currentLanguage['lng_ident']);
    if ($outputContent) {
      $serverUrl = PapayaUtilServerProtocol::get().'://'.PapayaUtilServerName::get();
      $url = strtr(
        $serverUrl.$this->papaya()->options->get('PAPAYA_PATH_WEB', '/'),
        '\\',
        '/'
      );
      $this->layout->setParam('PAGE_BASE_URL', $url);
      if (isset($_SERVER['REQUEST_URI']) && strpos($_SERVER['REQUEST_URI'], '?') > 0) {
        $url .= substr($_SERVER['REQUEST_URI'], 1, strpos($_SERVER['REQUEST_URI'], '?') - 1);
      } elseif (!empty($_SERVER['REQUEST_URI'])) {
        $url .= substr($_SERVER['REQUEST_URI'], 1);
      }
      $this->layout->setParam('PAGE_URL', $url);
      $this->layout->setParam(
        'PAGE_REQUEST_URL', (string)$this->papaya()->request->url
      );
      $this->layout->setParam(
        'PAGE_TITLE',
        $this->topic->topic['TRANSLATION']['topic_title']
      );
      if (defined('PAPAYA_WEBSITE_REVISION') && trim(PAPAYA_WEBSITE_REVISION) != '') {
        $this->layout->setParam('PAPAYA_WEBSITE_REVISION', PAPAYA_WEBSITE_REVISION);
        $this->layout->setParam('PAGE_WEBSITE_REVISION', PAPAYA_WEBSITE_REVISION);
      }
      $themeHandler = new PapayaThemeHandler();
      $this->layout->setParam('PAGE_THEME', $themeHandler->getTheme());
      $this->layout->setParam('PAGE_THEME_SET', $themeHandler->getThemeSet());
      $this->layout->setParam('PAGE_THEME_PATH', $themeHandler->getUrl());
      $this->layout->setParam('PAGE_THEME_PATH_LOCAL', $themeHandler->getLocalThemePath());
      $this->layout->setParam(
        'PAGE_WEB_PATH', $this->papaya()->options->get('PAPAYA_PATH_WEB', '/')
      );
      $this->layout->setParam(
        'PAGE_LANGUAGE',
        $this->topic->currentLanguage['lng_short']
      );
      $this->layout->setParam('PAGE_MODE_PUBLIC', $this->isPreview() ? 0 : 1);
      $this->layout->setParam('PAGE_OUTPUTMODE_CURRENT', $currentViewMode);
      $this->layout->setParam('PAGE_OUTPUTMODE_DEFAULT', $defaultViewMode);
      $this->layout->setParam(
        'PAGE_URL_LEVEL_SEPARATOR',
        $this->papaya()->options->get('PAPAYA_URL_LEVEL_SEPARATOR', '')
      );
      $this->layout->setParam('PAPAYA_TIME_OFFSET', date('O'));
      $this->layout->setParam(
        'PAPAYA_DBG_DEVMODE',
        $this->papaya()->options->get('PAPAYA_DBG_DEVMODE', FALSE)
      );
      $this->layout->setParam(
        'PAPAYA_DEBUG_LANGUAGE_PHRASES',
        $this->papaya()->options->get('PAPAYA_DEBUG_LANGUAGE_PHRASES', FALSE)
      );
      if (empty($_SERVER['HTTP_USER_AGENT']) ||
          $_SERVER['HTTP_USER_AGENT'] != 'mnogosearch-dimensional') {
        $boxesList = $this->getBoxes(
          $boxesId = $this->topic->getBoxesTopicId(),
          $boxGroupId = $this->topic->getBoxGroupsTopicId()
        );
        if ($boxesList) {
          $data = $boxesList->parsed(
            $boxesId,
            $this->topic->topic['TRANSLATION']['lng_id'],
            empty($this->output->viewMode['viewmode_id'])
               ? 0 : (int)$this->output->viewMode['viewmode_id']
          );
          if ($data) {
            $this->layout->add($data, 'boxes');
          }
        }
        unset($boxesList);
      }
      $this->topic->databaseClose();
    }
    return TRUE;
  }

  /**
   * Get the boxes list for the current page, load them if not already loaded
   */
  public function getBoxes($boxPageId = NULL, $boxGroupPageId = NULL) {
    if (!isset($this->_boxesList)) {
      if ($boxPageId || $boxGroupPageId) {
        if ($this->isPreview()) {
          include_once(PAPAYA_INCLUDE_PATH.'system/base_boxeslinks.php');
          $this->_boxesList = new base_boxeslinks($this->topic);
        } else {
          include_once(PAPAYA_INCLUDE_PATH.'system/papaya_publicboxeslinks.php');
          $this->_boxesList = new papaya_public_boxeslinks($this->topic);
        }
        $this->_boxesList->setPageId($boxPageId, $boxGroupPageId);
        $this->_boxesList->load(
          $this->topic->topic['TRANSLATION']['lng_id'],
          empty($this->output->viewMode['viewmode_id'])
            ? 0 : (int)$this->output->viewMode['viewmode_id']
        );
      } else {
        return FALSE;
      }
    }
    return $this->_boxesList;
  }

  /**
   * Returns the page xml document for the current request,
   * if document not yet generated, this will trigger it.
   *
   * @return DOMDocument
   */
  public function getPageDocument() {
    if (NULL === $this->_pageDocument) {
      $this->_pageDocument = new PapayaXmlDocument();
      $xml = PapayaUtilStringXml::repairEntities(
        $this->topic->parseContent(TRUE, $this->_filterOptions)
      );
      if (!empty($xml)) {
        $errors = new PapayaXmlErrors();
        $errors->activate();
        try {
          $this->_pageDocument->loadXml($xml);
          $errors->omit();
          $errors->deactivate();
        } catch (PapayaXmlException $e) {
          $message = new PapayaMessageLog(
            PapayaMessageLogable::GROUP_SYSTEM,
            PapayaMessage::TYPE_ERROR,
            $e->getMessage()
          );
          $message->context()->append(
            new PapayaMessageContextText($xml)
          );
          $this->papaya()->messages->dispatch($message);
          $errors->deactivate();
        }
      }
    }
    return $this->_pageDocument;
  }

  /**
  * Get xml wrapped output for a box without appling a template
  * @return string
  */
  function getBoxXML() {
    if (!$this->topic->topicExists($this->topicId)) {
      $this->topicId = $this->papaya()->options->get('PAPAYA_PAGEID_DEFAULT', 0);
    }
    $this->topic->loadOutput(
      $this->topicId, $this->requestData['language'], $this->versionDateTime
    );
    include_once(PAPAYA_INCLUDE_PATH.'system/papaya_boxes.php');
    $boxes = new papaya_boxes;
    $boxes->msgs = &$this->msgs;
    if ($boxes->load($this->boxId, $this->topic->topic['TRANSLATION']['lng_id'])) {
      $result = $boxes->parsedBox($this->topic);
      include_once(PAPAYA_INCLUDE_PATH.'system/sys_simple_xmltree.php');
      if (!simple_xmltree::isXML($result, $this)) {
        $result = '<box><![CDATA['.str_replace(']]>', ']]&gt;', $result).']]></box>';
      }
      return $result;
    }
    return '';
  }

  /**
  * Get box output after appling template if needed
  *
  * @return string
  */
  function getBox() {
    if (!$this->topic->topicExists($this->topicId)) {
      $this->topicId = $this->papaya()->options->get('PAPAYA_PAGEID_DEFAULT', 0);
    }
    $this->topic->loadOutput(
      $this->topicId, $this->requestData['language'], $this->versionDateTime
    );
    if ((isset($this->output->viewMode) && !empty($this->output->viewMode['viewmode_ext'])) ||
        $this->output->loadViewModeData($this->mode)) {
      if ($this->isPreview()) {
        include_once(PAPAYA_INCLUDE_PATH.'system/base_boxes.php');
        $boxes = new base_boxes;
      } else {
        include_once(PAPAYA_INCLUDE_PATH.'system/base_boxes_public.php');
        $boxes = new base_boxes_public;
      }
      $loaded = $boxes->load(
        $this->boxId, $this->papaya()->request->languageId
      );
      if ($loaded) {
        if ($this->validateDomain() &&
            $this->topic->checkPublishPeriod($this->topicId) &&
            $this->validateAccess($this->topicId)) {
          /**
           * @var PapayaResponse $response
           */
          $response = $this->papaya()->response;
          $response->setCache(
            $this->papaya()->session()->isActive() ? 'private' : 'public',
            $boxes->getBoxBrowserCacheTime($boxes->box)
          );
          $viewId = $boxes->getViewId();
          $useFilter = empty($boxes->box['TRANSLATION']['module_useoutputfilter'])
            ? FALSE : $boxes->box['TRANSLATION']['module_useoutputfilter'];
          if ($viewId > 0) {
            if ($this->filter = $this->output->getFilter($viewId)) {
              if (!$useFilter) {
                $str = $boxes->parsedBox($this->topic);
                $this->sendHeader('Last-modified: '.gmdate('D, d M Y H:i:s').' GMT');
                $this->output->sendHeader();
                $this->papaya()->session->close();
                return $str;
              } elseif ($this->filter->checkConfiguration()) {
                $xmlString = $boxes->parsedBox(
                  $this->topic,
                  $this->filter->data
                );
                if (!empty($xmlString)) {
                  $str = $this->filter->parseBox($this->topic, $boxes->box, $xmlString);
                } else {
                  $str = FALSE;
                }
                if (FALSE === $str) {
                  $this->papaya()->session->close();
                  echo $this->filter->errorMessage;
                  return FALSE;
                } else {
                  $this->sendHeader('Last-modified: '.gmdate('D, d M Y H:i:s').' GMT');
                  $this->output->sendHeader();
                  $this->papaya()->session->close();
                  return $str;
                }
              } else {
                $this->getError(
                  $this->filter->errorStatus,
                  'Output mode "'.
                    papaya_strings::escapeHTMLChars(basename($this->mode)).
                    '"for page #'.$this->topicId.
                    ' not available: '.$this->filter->errorMessage,
                  PAPAYA_PAGE_ERROR_OUTPUT_SETTINGS
                );
              }
            } else {
              $this->getError(
                500,
                'Output mode "'.
                  papaya_strings::escapeHTMLChars(basename($this->mode)).'" for page #'.
                  $this->topicId.' not found',
                PAPAYA_PAGE_ERROR_OUTPUT
              );
            }
          } else {
            $this->getError(
              500,
              'View "'.
                papaya_strings::escapeHTMLChars(basename($this->mode)).'" for page #'.
                $this->topicId.' not found',
              PAPAYA_PAGE_ERROR_VIEW
            );
          }
        } else {
          $this->getError(
            403,
            'Access forbidden',
            PAPAYA_PAGE_ERROR_ACCESS
          );
        }
      } else {
        $this->getError(
          404,
          'Box #"'.(int)$this->boxId.'" not found',
          PAPAYA_PAGE_ERROR_BOX
        );
      }
    } else {
      $this->getError(
        404,
        'Output mode "'.$this->mode.'" not found',
        PAPAYA_PAGE_ERROR_OUTPUT
      );
    }
  }

  /**
   * Execute output controller
   *
   * @param $controller
   * @return boolean Valid content | Error
   */
  function executeController(&$controller) {
    $counter = 10;
    do {
      $controller = &$controller->execute($this);
    } while (is_object($controller) &&
             --$counter > 0);
    return is_bool($controller) ? $controller : FALSE;
  }

  /**
  * Load media file data
  *
  * @param integer $id
  * @access public
  * @return mixed array $data or boolean
  */
  function loadMediaFileData($id, $version = NULL) {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_mediadb.php');
    $this->mediaDB = &base_mediadb::getInstance();
    if ($fileData = $this->mediaDB->getFile($id, $version)) {
      return $fileData;
    }
    return FALSE;
  }

  /**
  * Get media popup
  *
  * @access public
  */
  function getMediaPopup() {
    if ($file = $this->checkMediaPerm($this->requestData['media_id'])) {
      if ($file['FILETYPE'] >= 1 && $file['FILETYPE'] <= 3) {
        if (empty($file['file_title']) || trim($file['file_title']) == '') {
          $caption = $file['file_title'];
        } else {
          $caption = $file['file_name'];
        }
        $this->sendHeader('Content-type: text/html');
        printf(
          '<html><head><title>%s</title></head><body style="padding: 0px; margin: 0px;">',
          papaya_strings::escapeHTMLChars($caption)
        );
        printf(
          '<img src="%s" style="border: none; margin: 0px;" alt="%s" />',
          'image.media.'.($this->isPreview() ? 'preview.' : '').$file['file_id'],
          papaya_strings::escapeHTMLChars($caption)
        );
        print('</body></html>');
      } else {
        $this->logRequest();
        $this->outputFile($file['FILENAME'], $file);
      }
      exit;
    }
  }

  /**
  * Get media file
  *
  * @access public
  */
  function getMediaFile() {
    if (isset($this->requestData['media_id'])) {
      if ($file = $this->checkMediaPerm($this->requestData['media_id'])) {
        $this->logRequest();
        $this->outputFile($file['FILENAME'], $file);
      } else {
        $this->getError(404, 'File not found', PAPAYA_PAGE_ERROR_MEDIA_FILE);
      }
    }
  }

  /**
  * Get media tumbnail file
  *
  * @access public
  */
  function getMediaThumbFile($mediaId = NULL) {
    if (empty($mediaId)) {
      $mediaId = empty($this->requestData['media_id']) ? NULL : $this->requestData['media_id'];
    }
    if (!empty($mediaId)) {
      $mediaFilePattern = '~^
        (
         (
           (?:.*?([a-fA-F\d]{32})) # title, mode and id
           v([0-9]+) # version
         )
         (?:\.[0-9a-zA-Z]+)?
         (?:_([a-z]+))?_ #resize mode
         (\d+)x(\d+) #resize size
         (?:_([\da-fA-F]{32}))? # optional params
        )
        (\.[\w\d]+)? # extension
        $~x';
      if (preg_match($mediaFilePattern, $mediaId, $matches)) {
        // is thumbnail
        if ($file = $this->checkMediaPerm($matches[2])) {
          $path = '';
          for ($i = 0; $i < PAPAYA_MEDIADB_SUBDIRECTORIES; $i++) {
            $path .= $matches[1][$i].'/';
          }
          $fileName = PAPAYA_PATH_THUMBFILES.$path.$matches[1];
          if (isset($matches[9])) {
            $fileName .= $matches[9];
            $requestExt = strtolower($matches[9]);
          } else {
            $requestExt = '';
          }
          if (is_file($fileName)) {
            $file['file_size'] = filesize($fileName);
            if ($requestExt != '') {
              $fileNameExt = strrchr($file['file_name'], '.');
              $file['file_name'] = substr(
                $file['file_name'], 0, strlen($fileNameExt) * -1
              ).$requestExt;
              switch ($requestExt) {
              case '.gif' :
                $file['mimetype'] = 'image/gif';
                break;
              case '.jpg' :
                $file['mimetype'] = 'image/jpeg';
                break;
              case '.png' :
                $file['mimetype'] = 'image/png';
                break;
              }
            }
            if (isset($file['PUBLIC_FILE']) && $file['PUBLIC_FILE']) {
              $file['url'] = $this->_storageSetPublic(
                'thumbs',
                $matches[1].$matches[9],
                $file['mimetype']
              );
            }
            $this->logRequest();
            $this->outputFile($fileName, $file);
          } else {
            $this->getError(404, 'Thumbnail not found', PAPAYA_PAGE_ERROR_MEDIA_THUMBNAIL);
          }
        }
      } elseif ($file = $this->checkMediaPerm($mediaId)) {
        $this->logRequest();
        $this->outputFile($file['FILENAME'], $file);
      } else {
        $this->getError(404, 'Invalid FileId', PAPAYA_PAGE_ERROR_MEDIA);
      }
    } else {
      $this->getError(404, 'Empty FileId', PAPAYA_PAGE_ERROR_MEDIA);
    }
  }

  /**
  * Call setPublic() appropriately on the media file.
  *
  * @see PapayaMediaStorageService::setPublic
  * @param string $storageGroup
  * @param string $storageId
  * @param string $mimeType
  * @return string public url or empty
  */
  private function _storageSetPublic($storageGroup, $storageId, $mimeType) {
    $options = $this->papaya()->getObject('Options');
    $storage = PapayaMediaStorage::getService(
      $options->getOption('PAPAYA_MEDIA_STORAGE_SERVICE'),
      $options
    );
    if (!$storage->isPublic($storageGroup, $storageId, $mimeType)) {
      $storage->setPublic(
        $storageGroup,
        $storageId,
        TRUE,
        $mimeType
      );
      if (!$storage->isPublic($storageGroup, $storageId, $mimeType)) {
        return '';
      }
    }
    return $storage->getUrl($storageGroup, $storageId, $mimeType);
  }

  /**
  * get mime type of a file name
  * @param $fileName
  * @return string
  */
  function getFileMimeType($fileName) {
    $mediaDB = base_mediadb::getInstance();
    $properties = $mediaDB->getFileProperties($fileName, '');
    return $properties['mimetype'];
  }

  /**
  * Convert filename to mime type
  *
  * @param string $fileName
  * @access public
  * @return mixed string $mime['mimetype_text'] or NULL
  */
  function filenameToMimeType($fileName) {
    if (preg_match('/\.([\w\d]+)$/', $fileName, $regs)) {
      if (!(isset($this->mimeTypes) && is_object($this->mimeTypes))) {
        include_once(PAPAYA_INCLUDE_PATH.'system/base_mimetypes.php');
        $this->mimeTypes = new base_mimetypes();
      }
      if ($mime = $this->mimeTypes->loadMimeTypeByExt($regs[1])) {
        return $mime['mimetype_ext'];
      }
    }
    $agent = $this->getUserAgent();
    $mimeType = ($agent == 'IE' || $agent == 'OPERA') ?
      'application/octetstream' : 'application/octet-stream';
    return NULL;
  }


  /**
  * output download file
  *
  * @access public
  * @return mixed
  */
  function outputDownload() {
    if (isset($this->requestData['media_id'])) {
      if ($data = $this->checkMediaPerm($this->requestData['media_id'])) {
        $this->logRequest();
        $this->papaya()->session->close();
        include_once(PAPAYA_INCLUDE_PATH.'system/papaya_file_delivery.php');
        if (!papaya_file_delivery::outputDownload($data['FILENAME'], $data)) {
          $this->getError(404, 'Cannot read file', PAPAYA_PAGE_ERROR_MEDIA_READ);
        }
      } else {
        $this->getError(403, 'Permission denied', PAPAYA_PAGE_ERROR_ACCESS);
      }
    }
    $this->getError(404, 'File not found', PAPAYA_PAGE_ERROR_MEDIA_FILE);
  }

  /**
  * This method generates an XML output showing the metadata of a topic, i.e.
  * which outputs exist for this page.
  */
  function getOutputs() {
    $this->topic = $this->createPage();
    if ($this->topic->topicExists($this->topicId)) {
      if ($str = $this->getPageMetaXML()) {
        $this->sendHeader('Content-type: text/xml; charset: utf-8');
        echo $str;
      } else {
        $this->getError(
          404,
          'Page #'.(int)$this->topicId.' no content found',
          PAPAYA_PAGE_ERROR_PAGE_CONTENT
        );
      }
    } elseif ($this->topicId == -1) {
      $this->getError(
        404,
        'File/path '.$_SERVER['REDIRECT_URL'].' not found',
        PAPAYA_PAGE_ERROR_PATH
      );
    } else {
      $this->getError(
        404,
        'Page #'.(int)$this->topicId.' not found',
        PAPAYA_PAGE_ERROR_PAGE
      );
    }
  }

  /**
  * This method generates the XML representation of a page, i.e. the input of
  * the XSLT-template.
  */
  function getXMLOutput() {
    $this->topic = $this->createPage();
    if (!(PAPAYA_DBG_XML_OUTPUT && $this->validateEditorAccess())) {
      $this->getError(
        403,
        'Access forbidden',
        PAPAYA_PAGE_ERROR_ACCESS
      );
    } elseif (($this->boxId > 0)) {
      if ($str = $this->getBoxXML()) {
        $response = $this->papaya()->response;
        $response->setCache('nocache');
        $response->setContentType('text/xml', 'charset: utf-8');
        $response->content(new PapayaResponseContentString($str));
        $response->send();
      } else {
        $this->getError(
          404,
          'Box #'.(int)$this->boxId.' no content found',
          PAPAYA_PAGE_ERROR_BOX_CONTENT
        );
      }
    } elseif ($this->topic->topicExists($this->topicId)) {
      if ($str = $this->getPageXML()) {
        $this->sendHeader('Content-type: text/xml; charset: utf-8');
        echo $str;
      } else {
        $this->getError(
          404,
          'Page #'.(int)$this->topicId.' no content found',
          PAPAYA_PAGE_ERROR_PAGE_CONTENT
        );
      }
    } elseif ($this->topicId == -1) {
      $this->getError(
        404,
        'File/path '.$_SERVER['REDIRECT_URL'].' not found',
        PAPAYA_PAGE_ERROR_PATH
      );
    } else {
      $this->getError(
        404,
        'Page #'.(int)$this->topicId.' not found',
        PAPAYA_PAGE_ERROR_PAGE
      );
    }
  }

  /**
  * This method generates the output of a page.
  */
  function getPageOutput() {
    if ($this->papaya()->options->getOption('PAPAYA_LOG_RUNTIME_REQUEST', FALSE)) {
      PapayaRequestLog::getInstance()->logTime('Page defined');
    }
    $this->topic = $this->createPage();
    if ($this->boxId > 0) {
      /**
       * @var PapayaResponse $response
       */
      $response = $this->papaya()->response();
      if ($output = $this->getBox()) {
        $response->content(new PapayaResponseContentString((string)$output));
      }
      $response->send();
      $response->end();
    } elseif ($this->topic->topicExists($this->topicId)) {
      $loaded = $this->topic->loadOutput(
        $this->topicId,
        $this->requestData['language'],
        $this->versionDateTime
      );
      if (!$loaded) {
        $this->getError(404, 'Page not found', PAPAYA_PAGE_ERROR_PAGE);
      } elseif ((
                 isset($this->output->viewMode) &&
                 !empty($this->output->viewMode['viewmode_ext'])
                ) ||
                $this->output->loadViewModeData($this->mode)
               ) {
        $expires = $this->topic->getExpires();
        $this->sendCacheHeaders($this->isPreview() ? 0 : $expires);
        if ($this->useCache()) {
          //check cache
          if ($cacheId = $this->getCacheId()) {
            $this->sendHeader('ETag: "'.$cacheId.'"');
            //get cache
            if (!($this->getCache($cacheId))) {
              /* save the topic id because it may get changed through usage of
                 the symlink module */
              $topicId = $this->topicId;
              //generate page
              $page = $this->getPage();
              //set cache
              if ($this->topic->cacheable &&
                  $this->papaya()->response->getStatus() == 200) {
                $this->setCache($cacheId, $topicId, $page);
              }
            } else {
              $this->logRequest(
                $this->topic->languageIdentToId($this->requestData['language']),
                TRUE
              );
            }
          } else {
            $this->getPage();
          }
        } else {
          $this->getPage();
        }
      } else {
        $this->getError(
          404,
          'Output mode "'.$this->mode.'" not found',
          PAPAYA_PAGE_ERROR_OUTPUT
        );
      }
    } elseif ($this->topicId == -1) {
      $this->getError(
        404,
        'File/path '.$_SERVER['REDIRECT_URL'].' not found',
        PAPAYA_PAGE_ERROR_PATH
      );
    } else {
      $this->getError(
        404,
        'Page #'.(int)$this->topicId.' not found',
        PAPAYA_PAGE_ERROR_PAGE
      );
    }
    if ($this->papaya()->options->getOption('PAPAYA_LOG_RUNTIME_REQUEST', FALSE)) {
      PapayaRequestLog::getInstance()->omit();
    }
    return '';
  }

  /**
  * This method outputs a media file.
  *
  * @param string $fileName
  * @param array &$data
  * @access public
  */
  function outputFile($fileName, &$data) {
    $this->papaya()->session->close();
    include_once(PAPAYA_INCLUDE_PATH.'system/papaya_file_delivery.php');
    papaya_file_delivery::outputFile($fileName, $data);
    $this->getError(404, 'File not found', PAPAYA_PAGE_ERROR_MEDIA_FILE);
  }

  /**
  * This method returns the current microtime as a float number.
  *
  * @access private
  * @return float
  */
  function microtimeFloat() {
    list($usec, $sec) = explode(' ', microtime());
    return ((float)$usec + (float)$sec);
  }


  /**
  * This method checks, whether the current surfer is allowed to access a
  * specific media file.
  *
  * @todo apply changes to mediadb permission model
  * @param integer $mid
  * @access public
  * @return array $file
  */
  function checkMediaPerm($mid) {
    if (preg_match('~^([\da-fA-F]{32})v(\d+)(\.[0-9a-zA-Z]+)?$~', $mid, $matches)) {
      // normal new mediaID
      $file = $this->loadMediaFileData($matches[1], $matches[2]);
    } elseif (preg_match('~^([\da-fA-F]{32})$~', $mid, $matches)) {
      // normal new mediaID without a version
      $file = $this->loadMediaFileData($matches[1]);
    } elseif (preg_match('~^(.*)\.(.*)$~', $mid, $matches)) {
      // old mediaID
      $file = $this->loadMediaFileData($matches[1]);
    } else {
      return FALSE;
    }

    if ($file) {
      // Here it could be checked, whether a file is on the clipboard (folderid=-1)
      // and then allow or forbid the access.
      // Otherwise it could be set in getFolderPermissions.

      if (!$this->isPreview() || !$this->papaya()->administrationUser->isValid) {
        // if the file's folder has no surfer related permissions it may be returned
        $folderPermissions = $this->mediaDB->calculateFolderPermissions($file['folder_id']);
        if (!isset($folderPermissions['surfer_view']) &&
            !isset($folderPermissions['surfer_edit'])) {
          //public file - add status
          $file['PUBLIC_FILE'] = TRUE;
          $file['url'] = $this->_storageSetPublic(
            'files',
            $file['file_id'].'v'.$file['current_version_id'],
            $file['mimetype']
          );
          return $file;
        }
        // the surfer has one of the folder permissions
        $surfer = $this->papaya()->surfer;
        if ($surfer->hasOnePermOf(array_keys($folderPermissions['surfer_view']))) {
          return $file;
        }
      } elseif ($this->validateEditorAccess()) {
        return $file;
      }
    }
    // if we couldn't allow access, let the visitor know it
    $this->getError(403, 'Access forbidden to media file', PAPAYA_PAGE_ERROR_MEDIA_ACCESS, TRUE);
    return FALSE;
  }

  /**
  * This method determines the users browser examining the HTTP_USER_AGENT var.
  *
  * @access public
  * @return string OPERA or IE or STD
  */
  function getUserAgent() {
    if (empty($_SERVER['HTTP_USER_AGENT'])) {
      $agentString = '';
    } else {
      $agentString = strtolower($_SERVER["HTTP_USER_AGENT"]);
    }
    if (strpos($agentString, 'opera') !== FALSE) {
      return 'OPERA';
    } elseif (strpos($agentString, 'msie') !== FALSE) {
      return 'IE';
    } else {
      return 'STD';
    }
  }

  /**
  * This method checks, whether gzip compression may be used
  */
  function checkAcceptGzip() {
    if (isset($_SERVER['SERVER_PROTOCOL']) && $_SERVER['SERVER_PROTOCOL'] == 'HTTP/1.0') {
      $this->acceptGzip = FALSE;
      return;
    } else {
      $this->acceptGzip = function_exists('gzopen');
    }
    if ($this->acceptGzip && isset($_SERVER['HTTP_ACCEPT_ENCODING'])) {
      $gzipPattern = '~gzip\s*(;\s*q=(\d(\.\d{1,3})?))?~';
      if (preg_match($gzipPattern, $_SERVER['HTTP_ACCEPT_ENCODING'], $regs)) {
        if ((!isset($regs[1])) || ($regs[1] == '') || ((float)$regs[2] != 0)) {
          $this->acceptGzip = TRUE;
          return;
        }
      }
    }
    $this->acceptGzip = FALSE;
  }

  /**
  * This method logs the request to the statistic.
  *
  * @param integer $lngId optional, default value 0
  * @param boolean $cachedPage optional, default value FALSE
  * @access public
  */
  function logRequest($lngId = 0, $cachedPage = FALSE) {
    $this->papaya()->session->close();
    if ($this->papaya()->options->get('PAPAYA_PAGE_STATISTIC', FALSE)) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_statistic_logging.php');
      $statObj = new base_statistic_logging($this->msgs);
      if (!isset($this->requestData)) {
        $this->requestData = base_object::parseRequestURI();
      }
      $this->statisticRequestId =
        $statObj->logRequest($this->requestData, $lngId, $cachedPage);
    }
  }

  /**
  * This method logs the exit page of a request.
  *
  * @see base_statistic_logging::logRequest
  * @see base_statistic_logging::logExitPage
  * @param string $url
  * @access public
  */
  function logRequestExitPage($url) {
    $this->papaya()->session->close();
    if ($this->papaya()->options->get('PAPAYA_PAGE_STATISTIC', FALSE)) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_statistic_logging.php');
      $statObj = new base_statistic_logging($this->msgs);
      $this->statisticRequestId = $statObj->logRequest($this->requestData);
      $statObj->logExitPage($this->statisticRequestId, $url);
    }
  }


  /**
  * This method initializes $this->startTime with the current microtime
  *
  * @access public
  */
  function startTimer() {
    $this->startTime = microtime();
  }

  /**
  * This method returns the time passed since startTimer() was called.
  *
  * @param integer $digits optional, default value 2
  * @access public
  * @return float time passed with $digits decimal places
  */
  function stopTimer($digits = 2) {
    $to = explode(' ', microtime());
    $from = explode(' ', $this->startTime);
    $to = $to[0] + (double)$to[1];
    $from = $from[1] + (double)$from[0];
    return number_format(($to - $from) * 1000, $digits);
  }

  /**
  * This method outputs a debug msg
  *
  * @param array $msg
  * @access public
  * @return integer
  */
  function showDebug($msg) {
    echo '</form>', '</table><br>';
    echo '<table style="position: relative; z-index: 10000;"'.
         ' border="1" width="800" cellpadding="2" cellspacing="0"'.
         ' align="center" bgcolor="#C0C0C0">';
    echo sprintf(
      '<tr><th>Debugausgabe (%s)</th><th align="right">'.
      $msg['time'].' ms</th></tr>',
      'Gesamtlaufzeit'
    );
    echo '</table><br/>';
    flush();
    return 0;
  }

  /**
  * This method handles errors, as it redirects accordingly or generates an error output.
  *
  * @param integer $error
  * @param string $errorString
  * @param boolean $verbose optional, default value FALSE
  * @access public
  */
  function getError($error, $errorString, $errorCode = NULL, $verbose = FALSE) {
    if (!$verbose) {
      $this->mode = $this->papaya()->options->get('PAPAYA_URL_EXTENSION', 'html');
      $this->topic = $this->createPage();
      if (empty($this->requestData['language'])) {
        $lngIdent = '';
      } else {
        $lngIdent = $this->requestData['language'];
      }
      switch ($error) {
      case 404:
        $errorStatus = 404;
        $this->topic->loadOutput(
          $this->papaya()->options->get('PAPAYA_PAGEID_ERROR_404', 0), $lngIdent
        );
        break;
      case 403:
        $errorStatus = 403;
        $this->topic->loadOutput(
          $this->papaya()->options->get('PAPAYA_PAGEID_ERROR_403', 0), $lngIdent
        );
        break;
      default:
        $errorStatus = 500;
        $this->topic->loadOutput(
          $this->papaya()->options->get('PAPAYA_PAGEID_ERROR_500', 0), $lngIdent
        );
      }
      $this->error = array(
        'status' => $errorStatus,
        'code' => $errorCode,
        'message' => $errorString
      );
      if ((empty($_GET['redirect']) || $_GET['redirect'] != $errorStatus) &&
          $this->topic->topicId) {
        $targetUrl = $this->getAbsoluteURL(
          $this->papaya()->options->get('PAPAYA_PATH_WEB', '/'),
          $this->getBaseLink()
        );
        $targetUrl .=
          '?redirect='.$errorStatus.
          '&msg='.urlencode($errorString).
          '&code='.(int)$errorCode;

        $protocol = PapayaUtilServerProtocol::get();
        $currentUrl = $protocol.'://'.PapayaUtilServerName::get().$_SERVER['REQUEST_URI'];

        include_once(PAPAYA_INCLUDE_PATH.'system/base_url_analyze.php');
        if (base_url_analyze::comparePathDepth($targetUrl, $currentUrl) !== 0) {
          $this->doRedirect(302, $targetUrl, 'Error Redirect');
        } else {
          $this->sendHTTPStatus($errorStatus);
        }
      } else {
        $this->sendHTTPStatus($errorStatus);
      }
      if ($this->topic->topicId) {
        if ($this->topic->checkPublishPeriod($this->topic->topicId)) {
          if ($this->validateAccess($this->topic->topicId)) {
            $viewId = $this->topic->getViewId();
            if ($viewId > 0) {
              if (!(isset($this->output) && is_object($this->output))) {
                include_once(PAPAYA_INCLUDE_PATH.'system/papaya_output.php');
                $this->output = new papaya_output;
              }
              $this->output->loadViewModeData($this->mode);
              if ($this->filter = $this->output->getFilter($viewId)) {
                $this->generatePage($this->filter->data, TRUE, FALSE);
                if ($this->filter->checkConfiguration()) {
                  $str = $this->filter->parsePage($this->topic, $this->layout);
                  $this->sendHeader('Last-modified: '.gmdate('D, d M Y H:i:s').' GMT');
                  $this->output->sendHeader();
                  $this->sendHeader('Content-length: '.strlen($str));
                  print $str;
                  exit();
                } else {
                  $this->getErrorHTML(
                    $this->filter->errorStatus,
                    'Output mode "'.
                      papaya_strings::escapeHTMLChars(basename($this->mode)).'"for page #'.
                      $this->topic->topicId.' not available: '.$this->filter->errorMessage,
                    PAPAYA_PAGE_ERROR_OUTPUT_SETTINGS
                  );
                }
              } else {
                $this->getErrorHTML(
                  500,
                  'Output mode "'.
                    papaya_strings::escapeHTMLChars(basename($this->mode)).'" for page #'.
                    $this->topic->topicId.' not found',
                  PAPAYA_PAGE_ERROR_OUTPUT
                );
              }
            } else {
              $this->getErrorHTML(
                500,
                  'View "'.
                  papaya_strings::escapeHTMLChars(basename($this->mode)).'" for page #'.
                  $this->topic->topicId.' not found',
                PAPAYA_PAGE_ERROR_VIEW
              );
            }
          } else {
            $this->getErrorHTML(403, 'Access forbidden', PAPAYA_PAGE_ERROR_ACCESS);
          }
        } else {
          $this->getErrorHTML(
            404,
            sprintf('Error page #%d currently not published', (int)$this->topicId),
            PAPAYA_PAGE_ERROR_PAGE_PUBLIC
          );
        }
      } else {
        $this->getErrorHTML($error, $errorString, $errorCode);
      }
    } else {
      $this->getErrorHTML($error, $errorString, $errorCode);
    }
  }

  /**
   * This method generates a simple default error HTML output.
   *
   * @param integer $status
   * @param string $errorString
   * @param integer|string $errorCode
   */
  function getErrorHTML($status, $errorString, $errorCode) {
    $controller = new PapayaControllerError($this);
    $controller->setStatus($status);
    $controller->setError($errorCode, $errorString);
    $controller->execute($this);
    exit;
  }

  /**
  * This method generates an informational redirection HTML page, that enables the visitor
  * to go to the requested page while informing him about possible security issues.
  *
  * It can use a defined papaya page for this or use a simple default HTML output generated
  * by getRedirectHTML()
  *
  * @param integer $status the redirect status code
  * @param string $url the URL to be redirected to
  * @access public
  */
  function getRedirect($status, $url) {
    $this->mode = $this->papaya()->options->get('PAPAYA_URL_EXTENSION', 'html');
    $this->topic = $this->createPage();
    if (empty($this->requestData['language'])) {
      $lngIdent = '';
    } else {
      $lngIdent = $this->requestData['language'];
    }
    switch ($status) {
    case 302:
      $this->topic->loadOutput((int)PAPAYA_PAGEID_STATUS_302, $lngIdent);
      break;
    case 301:
    default :
      $status = 301;
      $this->topic->loadOutput((int)PAPAYA_PAGEID_STATUS_301, $lngIdent);
      break;
    }
    $this->redirect = array(
      'status' => $status,
      'url' => $url
    );
    $GLOBALS['PAPAYA_PAGE'] = &$this;
    if ($this->topic->topicId) {
      if ($this->topic->checkPublishPeriod($this->topic->topicId)) {
        if ($this->validateAccess($this->topic->topicId)) {
          $viewId = $this->topic->getViewId();
          if ($viewId > 0) {
            if (!(isset($this->output) && is_object($this->output))) {
              include_once(PAPAYA_INCLUDE_PATH.'system/papaya_output.php');
              $this->output = new papaya_output;
            }
            $this->output->loadViewModeData($this->mode);
            if ($this->filter = $this->output->getFilter($viewId)) {
              $this->generatePage($this->filter->data, TRUE, FALSE);
              if ($this->filter->checkConfiguration()) {
                $str = $this->filter->parsePage($this->topic, $this->layout);
                $this->sendHeader('Last-modified: '.gmdate('D, d M Y H:i:s').' GMT');
                $this->output->sendHeader();
                print $str;
                exit();
              } else {
                $this->getErrorHTML(
                  $this->filter->errorStatus,
                  'Output mode "'.
                    papaya_strings::escapeHTMLChars(basename($this->mode)).'"for page #'.
                    $this->topic->topicId.' not available: '.$this->filter->errorMessage,
                  PAPAYA_PAGE_ERROR_OUTPUT_SETTINGS
                );
              }
            } else {
              $this->getErrorHTML(
                500,
                'Output mode "'.
                  papaya_strings::escapeHTMLChars(basename($this->mode)).'" for page #'.
                  $this->topic->topicId.' not found',
                PAPAYA_PAGE_ERROR_OUTPUT
              );
            }
          } else {
            $this->getErrorHTML(
              500,
                'View "'.
                papaya_strings::escapeHTMLChars(basename($this->mode)).'" for page #'.
                $this->topic->topicId.' not found',
              PAPAYA_PAGE_ERROR_VIEW
            );
          }
        } else {
          $this->getErrorHTML(403, 'Access forbidden', PAPAYA_PAGE_ERROR_ACCESS);
        }
      } else {
        $this->getErrorHTML(
          404,
          sprintf('Error page #%d currently not published', (int)$this->topicId),
          PAPAYA_PAGE_ERROR_PAGE_PUBLIC
        );
      }
    } else {
      $this->getRedirectHTML($status, $url);
    }
  }

  /**
  * This method generates a simple redirect HTML page, if no papaya page is defined.
  *
  * @param integer $status the redirect status code
  * @param string $url the URL to be redirected to
  */
  function getRedirectHTML($status, $url) {
    $this->sendHTTPStatus(200);
    $this->sendHeader('Content-type: text/html; charset=utf-8');
    printf(
      '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">
      <html>
      <head>
      <title>Redirect - %1$d</title>
      <style type="text/css">
      <!--
        body {
          background-color: #FFF;
          font-family: Verdana, Geneva, Arial, Helvetica, sans-serif;
        }
        th, td {
          font-family: Verdana, Geneva, Arial, Helvetica, sans-serif;
        }
        a {
          color: #95A41A;
        }
      //-->
      </style>
      </head>
      <body>
      <div align="center">
      <br />
      <br />
      <br />
      <table cellpadding="4" border="0" width="400">
        <tr>
          <th colspan="2">The page you requested is on another domain!
            Please click the link below to proceed to the requested page.</th>
        </tr>
        <tr valign="top">
          <td align="center"><a href="%2$s">%2$s</a></td>
        </tr>
      </table>
      </div>
      </body>
      </html>',
      (int)$status,
      papaya_strings::escapeHTMLChars(papaya_strings::ensureUTF8($url))
    );
    exit();
  }

  /**
  * Send HTTP status
  *
  * @param integer $status
  * @access public
  */
  function sendHTTPStatus($status = NULL) {
    $response = $this->papaya()->response;
    if (isset($status)) {
      $response->setStatus($status);
    }
    $response->sendStatus();
  }

  /**
  * This method sends out a string as header()
  *
  * @param string $headerString the string to be send as header
  * @param boolean $replace replace existing header
  */
  function sendHeader($headerStr, $replace = TRUE) {
    static $headersSent;
    $file = '';
    $line = '';
    if (isset($headersSent) && $headersSent) {
      //already sent some headers and reported an error
      return FALSE;
    } elseif ($headersSent = headers_sent($file, $line)) {
      //report the error to log
      $errorMsg = 'WARNING #2 Cannot modify header information - headers already sent';
      $this->logMsg(
        MSG_WARNING,
        PapayaMessageLogable::GROUP_PHP,
        $errorMsg,
        $errorMsg.' in '.$file.':'.$line,
        TRUE,
        2
      );
      return FALSE;
    } else {
      if ($this->papaya()->options->get('PAPAYA_DISABLE_XHEADERS', FALSE) &&
          strtoupper(substr($headerStr, 0, 2)) == 'X-') {
        return TRUE;
      }
      header($headerStr, $replace);
    }
    return TRUE;
  }

  /**
  * Validate if the request page/box should contain the public content
  *
  * For now the current status is assigned to the old $public member variable for bc, too.
  *
  * @return boolean
  */
  public function isPreview() {
    if (isset($this->_isPreview)) {
      return $this->_isPreview;
    }
    $this->_isPreview = $this->papaya()->request->getParameter(
      'preview', FALSE, NULL, PapayaRequest::SOURCE_ALL
    );
    $this->papaya()->pageReferences->setPreview($this->_isPreview);
    $this->public = !$this->_isPreview;
    return $this->_isPreview;
  }

  /**
  * Return the current domain id, this can be set after handleDomain() and changed id
  * a preview domain is found in the session.
  *
  * @return boolean
  */
  public function getCurrentDomainId() {
    return $this->_currentDomainId;
  }
}

